% File: DOG.m
% Purpose: Difference of Gaussians computations
% Parameters:
%   size: size of the filter
%   sigma_y: height of the Guassian
%   sigma_x1: width of Gaussan #1
%   sigma_x2: width of Gaussan #2
%   theta: orientation
%   x0 and y0: are the center of the Gaussians
% Returns:
%   dog: real part of the filter
%   doog: phase
%
% Author: Antonio J. Rodriguez-Sanchez
%
% CVR, Intelligent and Interactive Systems
% Copyright (c) 2012, York University/University of Innsbruck
%
% Permission is granted to copy and distribute this file, for noncommercial
% use, provided (a) this copyright notice is preserved, (b) no attempt
% is made to restrict redistribution of this file, and (c) this file is
% restricted by a compilation copyright.
%
%  All comments concerning this program may be sent to the
%  e-mail address 'Antonio.Rodriguez-Sanchez@uibk.ac.at'
%

function [dog doog] = DOG(size,sigma_y,sigma_x1,sigma_x2,theta,x0,y0)
 
[X Y]=meshgrid(-fix(size/2):fix(size/2),fix(-size/2):fix(size/2));

% Rotation 
x_theta=(X-x0)*cos(theta)+(Y-y0)*sin(theta);
y_theta=-(X-x0)*sin(theta)+(Y-y0)*cos(theta);

dog1 = 1/(2*pi*sigma_x1*sigma_y).*...
    exp(-.5*(x_theta.^2/sigma_x1^2+y_theta.^2/sigma_y^2));
dog2 = 1/(2*pi*sigma_x2*sigma_y).*...
    exp(-.5*(x_theta.^2/sigma_x2^2+y_theta.^2/sigma_y^2));

dog = dog1-dog2;

x_theta = x_theta+sigma_y/2;
doog2 = 1/(2*pi*sigma_x1*sigma_y).*...
    exp(-.5*(x_theta.^2/sigma_x1^2+y_theta.^2/sigma_y^2));
doog = dog1-doog2;
