% File: V4.m
% Purpose: V4 layer->local curvature and shape cells responses
% Parameters:
%   - laV1: structure containing V1 neuron responses
%   - laV2: structure containing V1 neuron responses
%   - m and mc are the max curvature values obtained from Cal_endsc
%   - center: Use centroid of shape or center of neuron
%   - plt: plot
% Returns:
%   V2: struct with simple and endstopped neuron values.
%    - act is the maximum values,
%      these are not used but left just in case is needed in the future
%    - curv: local curvature values
%    - resp: shape neuron response
%
% Author: Antonio J. Rodriguez-Sanchez
%
% CVR, Intelligent and Interactive Systems
% Copyright (c) 2012, York University/University of Innsbruck
%
% Permission is granted to copy and distribute this file, for noncommercial
% use, provided (a) this copyright notice is preserved, (b) no attempt
% is made to restrict redistribution of this file, and (c) this file is
% restricted by a compilation copyright.
%
%  All comments concerning this program may be sent to the
%  e-mail address 'Antonio.Rodriguez-Sanchez@uibk.ac.at'
%

function V4 = V4(laV1, laV2, m, mc, center, plt)

ndir = size(laV2.endsc,1);
ncurv = size(laV2.endsc,2)*2;
n1 = size(laV2.endsc,3); 
n2 = size(laV2.endsc,4); 
% -100 is temporal for efficiency fot the stimuli
% REMOVE for other stimuli
rad = 0:10:(max(n1,n2)-100)/2*sqrt(2);
or = -pi/45:pi/45:2*pi+pi/45;

V4 = struct('act', zeros(ncurv,length(rad),length(or)), ...
    'curv', zeros(length(rad),length(or)), ...
    'resp', 0);

x = repmat(1:n2,[length(1:n1) 1]);
y = repmat(1:n1,[length(1:n2) 1]);
y = y';

for j=1:length(m)
  laV2.endsc(:,j,:,:) = Rectify(laV2.endsc(:,j,:,:),0.01,m(j)/8.5);
  laV2.corner(:,j,:,:) = Rectify(laV2.corner(:,j,:,:),0.01,mc(j)/8.5);
  laV1.simple_e(:,j,:,:) = Rectify(laV1.simple_e(:,j,:,:),0.01,max(max(max(laV1.simple_e(:,j,:,:))))/8.5);
end

% c_class: 1 -> sharp curv
%          2-4 -> curv type, pos
%          5 -> straight line
%          6-8 -> curv type, neg
% this can be modified to incorporate corner neurons
% un comment belown, then it would be
% c_class: 1 -> corner neurons
%          2-5 -> curv type, pos
%          6 -> straight line
%          7-10 -> curv type, neg
%          11 -> corner neurons

sp = laV2.signp>laV2.signn; sn = laV2.signn>laV2.signp;
c_class = zeros(ndir,ncurv,n1,n2);
corner = repmat(laV2.corner(1,:,:,:),[12 1]);

% aux = zeros(size(laV2.endsc)); aux(sp>0.5) = corner(sp>0.5);
% c_class(:,1,:,:) = aux(:,3,:,:);
c_class(:,1,:,:) = laV2.endsc(:,1,:,:);
aux = zeros(size(laV2.endsc)); aux(sp) = laV2.endsc(sp);
c_class(:,2:4,:,:) = aux(:,2:4,:,:);
% r = laV2.endsc<0.2 & laV1.simple_e>corner;
% aux = zeros(size(laV2.endsc)); 
% aux(r) = laV1.simple_e(r);
% c_class(:,6,:,:) = sum(aux,2)*2/(ncurv-1);
r = all(laV2.endsc<0.25,2) & all(laV1.simple_e>corner,2);
c_class(:,5,:,:) = max(laV1.simple_e,[],2).*r;
aux = zeros(size(laV2.endsc)); aux(sn) = laV2.endsc(sn);
c_class(:,8:-1:6,:,:) = aux(:,2:4,:,:);
% aux = zeros(size(laV2.endsc)); aux(sn) = corner(sn);
% c_class(:,11,:,:) = aux(:,3,:,:);

prob = reshape(max(c_class,[],1),[ncurv n1 n2]);
% prob = reshape(sum(c_class,1),[ncurv n1 n2]);
% prob = reshape(max(laV2.endsc,[],1),[ndir n1 n2]);
% prob = permute(prob,[3 4 2 1]);
[maxs inds] = max(prob,[],1);
inds(maxs<0.5) = 0;

theta = atan2(center(2)-y,x-center(1));
theta(theta<0) = theta(theta<0)+2*pi;
d = sqrt((y-center(2)).^2+(x-center(1)).^2);

for i=2:length(rad)
  for j=2:length(or)
    mat = theta>=or(j-1) & theta<=or(j);
    mat = mat & (d>=rad(i-1) & d<=rad(i));
    for z=1:ncurv
      V4.act(z,i-1,j-1) = sum(prob(z,mat));
    end
  end
end

V4.resp = sum(sum(sum(V4.act)));

if(plt)
  for i=1:size(inds,2)
    for j=1:size(inds,3)
      if((any(theta(j,i)>=or-.01 & theta(j,i)<=or+.01)) | ... 
          (any(d(j,i) >= rad-.5 & d(j,i) <= rad+.5)))
            inds(1,j,i) = 1;
      end
    end
  end 
  % figure, imagesc(reshape(inds,[n1 n2])); colormap(gray); 
end;
