% File: main.m
% Purpose: This file runs the program
%
% Author: Antonio J. Rodriguez-Sanchez
%
% CVR, Intelligent and Interactive Systems
% Copyright (c) 2012, York University/University of Innsbruck
%
% Permission is granted to copy and distribute this file, for noncommercial
% use, provided (a) this copyright notice is preserved, (b) no attempt
% is made to restrict redistribution of this file, and (c) this file is
% restricted by a compilation copyright.
%
%  All comments concerning this program may be sent to the
%  e-mail address 'Antonio.Rodriguez-Sanchez@uibk.ac.at'
%

clear;

ndir = 12; % ndir orientations
ncurv = 8; % ncurv curvatures
plt = 0; % Plot, 1=yes, 0=no plot
cent = 0; % Use object centroid (1) or center of neuron (0)

% not all stimuli are at 8 orientations ...
count = [ 1 1 8 8 4 8 4 ...
  8 8 8 8 8 8 8 ...
  8 8 8 8 8 8 8 ...
  8 8 8 8 8 8 8 ...
  8 8 8 2 8 4 8 ...
  8 2 8 ...
  8 8 8 8 8 8 8 ...
  8 8 8 8 8 8];

% These are good values for the displaced ES model

sigma_y = [10 15 22 30];
WR = 2.5;
AR = [.7*WR 1.4*WR 2.15*WR 3*WR];
fac = [1.5 1.25 1 3];

for i=1:length(sigma_y)
  fesc(i) = struct('sigma_y', sigma_y(i), 'size', 4*sigma_y(i)+1, ...
    'sigma_x1', sigma_y(i)/AR(i), 'sigma_x2', WR*sigma_y(i)/AR(i), ...
    'pr', 1.5, 'x0', 0, 'y0', 0);
end

% These values are obtained after Calibration using the parameter values before.
% Use Cal_endsc.m to test other values
% e.g.: [m ms mc l h] = Cal_endsc(fesc, fac, 12, 0);

m = [9.4959; 8.7135; 6.8242; 28.734];
mc = [123.1804; 82.6394; 58.4508; 45.2915];

% Datafileneurons.txt contains a list corresponding to the 
% V4 neuron datafiles
% They are not provided in this package
% YOU NEED TO ASK Dr. PASUPATHY (U. of Washington) FOR THESE DATA FILES
fneuron = fopen('Datafileneurons.txt');
fresults = fopen('Results.txt','w');

% Call the function corresponding to the file
while ~feof(fneuron)
    fdata1 = fgetl(fneuron);
    fdata = strrep(fdata1, '.', '');
    func = str2func(fdata);
    [error stdev] = feval(func, ndir, ncurv, fac, count, fesc, m, mc, cent, plt);
    fprintf(fresults,'%s - error: %2.2f, stdev: %2.2f\n', fdata1, error, stdev);  
end

fclose(fneuron);
fclose(fresults);
